@@ -1,4 +1,5 @@
 # -*- coding: UTF-8 -*-
+from weakref import WeakSet
 from django.conf.urls import patterns, url, include
 from django.core.exceptions import ImproperlyConfigured
 from django.core.urlresolvers import reverse
@@ -30,10 +31,47 @@ except ImportError:
     
 
 
+class FlowHandlerBase(object):
+    registry = WeakSet()
 
-class FlowHandler(object):
-    
-    def __init__(self, app_namespace=None, state_store=None):
+    def __init__(self, *args, **kwargs):
+        FlowHandlerBase.registry.add(self)
+        super(FlowHandlerBase, self).__init__(*args, **kwargs)
+
+    @staticmethod
+    def full_flow_graph(request):
+        """
+        This is a Django view function that returns a PNG that has all flows
+        of all `FlowHandler`s created in a Django shell. It can do that because
+        `FlowHandlerBase` keeps track of all `FlowHandler` instances created.
+        Basically it combines all .flowgraph debug URLs. To use it, just
+        add the handler to your URL config like this:
+
+            if settings.DEBUG:
+                urlpatterns += patterns('',
+                    url(r'^\.fullflowgraph$', FlowHandlerBase.flow_graph),
+                )
+        """
+        logger.debug('(%s) flow_graphs' % len(FlowHandlerBase.registry))
+        if not has_pydot:
+            raise ImproperlyConfigured('The pydot library is required to see flowgraph debug output')
+
+        graph = pydot.Dot(graph_type='graph')
+
+        for handler in FlowHandlerBase.registry:
+            logger.debug('--- %s' % handler)
+            for flow in handler._entry_points:
+                handler._add_flow_nodes(graph, flow)
+
+        data = graph.create_png()
+
+        return HttpResponse(data, content_type='image/png')
+
+
+class FlowHandler(FlowHandlerBase):
+
+    def __init__(self, app_namespace=None, state_store=None, *args, **kwargs):
+        super(FlowHandler, self).__init__(*args, **kwargs)
         self._entry_points = []
         self.app_namespace = app_namespace
         self.state_store = state_store or default_state_store
@@ -168,7 +206,8 @@ class FlowHandler(object):
         return HttpResponse(data, content_type='image/png')
         
     def flow_entry_link(self, request, flow_class_or_name, on_complete_url=None,
-                              with_state=False, flow_namespace=None, url_args=None, url_kwargs=None):
+                              with_state=False, flow_namespace=None, url_args=None, url_kwargs=None,
+                              url_queryargs=None):
 
         flow_class = get_by_class_or_name(flow_class_or_name)
         
@@ -180,12 +219,14 @@ class FlowHandler(object):
             state = {'_id': ''} # TODO: this is a bit of a hack, but task_id is required...
         instance = position.create_instance(state, self.state_store, url_args=url_args, url_kwargs=url_kwargs)
         
-        url = instance.get_absolute_url(include_flow_id=False)
+        inst_url = instance.get_absolute_url(include_flow_id=False)
         
-        parts = urlparse.urlparse(url)
+        parts = urlparse.urlparse(inst_url)
         query = urlparse.parse_qs(parts.query)
         if on_complete_url is not None:
             query['_on_complete'] = on_complete_url
+        if url_queryargs is not None:
+            query.update(url_queryargs)
         parts = list(parts)
         parts[4] = urllib.urlencode(query)
         
@@ -214,7 +255,8 @@ class FlowHandler(object):
         for flow in self._entry_points:
             urlpatterns += self._urls_for_flow(flow_namespace, flow)
         if settings.DEBUG:
-            urlpatterns += patterns('', url('.flowgraph$', self.flow_graph))
+            prefix = flow_namespace if flow_namespace else ''
+            urlpatterns += patterns('', url('%s\.flowgraph$' % prefix, self.flow_graph))
             
         # verify that URLs are unique
         url_list = self.list_urls(urlpatterns)
@@ -403,9 +445,9 @@ class FlowPositionInstance(object):
             
             elif isinstance(response, Action):
                 # this is a new action for the user, so redirect to it
-                url = response.get_absolute_url()
-                response = redirect(url)
-                
+                absurl = response.get_absolute_url()
+                response = redirect(absurl)
+               
             elif isinstance(response, basestring):
                 # this is a string which should be the name of an action
                 # which couldn't be referenced as a class for some reason
